# -*- coding: utf-8 -*-
from __future__ import unicode_literals


"""
id custom locale file.
"""

translations = {
    "units": {"few_second": "beberapa detik"},
    "ago": "{} yang lalu",
    "from_now": "dalam {}",
    "after": "{0} kemudian",
    "before": "{0} yang lalu",
    "date_formats": {
        "LTS": "HH:mm:ss",
        "LT": "HH:mm",
        "LLLL": "dddd [d.] D. MMMM YYYY HH:mm",
        "LLL": "D. MMMM YYYY HH:mm",
        "LL": "D. MMMM YYYY",
        "L": "DD/MM/YYYY",
    },
}
