# -*- coding: utf-8 -*-
"""
it custom locale file.
"""

from __future__ import unicode_literals


translations = {
    "units": {"few_second": "alcuni secondi"},
    # Relative Time
    "ago": "{0} fa",
    "from_now": "in {0}",
    "after": "{0} dopo",
    "before": "{0} prima",
    # Ordinals
    "ordinal": {"other": "°"},
    # Date formats
    "date_formats": {
        "LTS": "H:mm:ss",
        "LT": "H:mm",
        "L": "DD/MM/YYYY",
        "LL": "D MMMM YYYY",
        "LLL": "D MMMM YYYY [alle] H:mm",
        "LLLL": "dddd, D MMMM YYYY [alle] H:mm",
    },
}
